import { Subject, Observable, Subscription } from 'rxjs';
export { toggleMap } from './toggleMap';
export { Subscription } from 'rxjs';
export interface Event {
    type: string;
    payload?: any;
    error?: boolean;
    meta?: Object;
}
export interface EventWithAnyFields extends Event {
    [others: string]: any;
}
export interface EventWithResult extends Event {
    result?: Promise<any>;
}
export interface Predicate {
    (item: Event): boolean;
}
export declare type EventMatcher = string | string[] | RegExp | Predicate | boolean;
/**
 * A Filter runs a synchronous function prior to any listeners
 * being invoked, and can cancel future filters, and all listeners
 * by throwing an exception, which must be caught by the caller of
 * `trigger`.
 *
 * It does *not*, as its name suggest, split off a slice of the
 * stream. To do that see `query`.
 * @see query
 */
export interface Filter<T> {
    (item: T): any;
}
/**
 * The function you assign to `.on(eventType, fn)`
 * events is known as a Listener. It receives the event.
 */
export interface Listener<T> {
    (item: T): any;
}
/**
 * When a listener is async, returning an Observable, it's possible
 * that a previous Observable of that listener is running already.
 * Concurrency modes control how the new and old listener are affected
 * when when they overlap.
 *
 * ![concurrency modes](https://s3.amazonaws.com/www.deanius.com/ConcurModes2.png)
 */
export declare enum ConcurrencyMode {
    /**
     * Newly returned Observables are subscribed immediately, without regard to resource constraints, or the ordering of their completion. (ala mergeMap) */
    parallel = "parallel",
    /**
     * Observables are enqueued and always complete in the order they were triggered. (ala concatMap)*/
    serial = "serial",
    /**
     * Any existing Observable is canceled, and a new is begun (ala switchMap) */
    replace = "replace",
    /**
     * Any new Observable is not subscribed if another is running. (ala exhaustMap) */
    ignore = "ignore",
    /**
     * Any new Observable is not subscribed if another is running, and
     * the previous one is canceled. (ala switchMap with empty() aka toggleMap) */
    toggle = "toggle"
}
export interface TriggerConfig {
    next?: string;
    complete?: string;
    error?: string;
}
export interface ListenerConfig {
    /** The concurrency mode to use. Governs what happens when another handling from this handler is already in progress. */
    mode?: ConcurrencyMode | 'serial' | 'parallel' | 'replace' | 'ignore' | 'toggle';
    /** A declarative way to map the Observable returned from the listener onto new triggered events */
    trigger?: TriggerConfig | true;
    takeUntil?: EventMatcher;
}
export interface TriggererConfig {
    result?: EventMatcher;
}
export declare class Channel {
    private channel;
    private filters;
    private listeners;
    private listenerEnders;
    private listenerParts;
    errors: Subject<string | Error>;
    private logs;
    constructor();
    trigger<T>(eventOrType: string | (EventWithAnyFields & T), payload?: T, config?: TriggererConfig): EventWithResult;
    /**
     * Provides an Observable of matching events from the channel.
     */
    query<T extends Event>(eventMatcher: EventMatcher, payloadMatcher?: Object): Observable<T>;
    filter<T extends Event>(eventMatcher: EventMatcher, f: Filter<T>): Subscription;
    listen<T extends Event>(eventMatcher: EventMatcher, listener: Listener<T>, config?: ListenerConfig): Subscription;
    on(eventMatcher: EventMatcher, listener: Listener<Event>, config?: ListenerConfig): Subscription;
    spy(listener: Listener<Event>): Subscription;
    /** Logs to the console (but not in test mode) */
    log(...args: any[]): void;
    reset(): void;
    private deactivateListener;
}
export declare const channel: Channel;
export declare const trigger: <T>(eventOrType: string | (EventWithAnyFields & T), payload?: T | undefined, config?: TriggererConfig | undefined) => EventWithResult;
export declare const query: <T extends Event>(eventMatcher: EventMatcher, payloadMatcher?: Object | undefined) => Observable<T>;
export declare const filter: <T extends Event>(eventMatcher: EventMatcher, f: Filter<T>) => Subscription;
export declare const listen: <T extends Event>(eventMatcher: EventMatcher, listener: Listener<T>, config?: ListenerConfig) => Subscription;
export declare const on: (eventMatcher: EventMatcher, listener: Listener<Event>, config?: ListenerConfig) => Subscription;
export declare const spy: (listener: Listener<Event>) => Subscription;
export declare const log: (...args: any[]) => void;
export declare const reset: () => void;
